#!/usr/bin/env python
import re
from collections import OrderedDict

def readTrimalOut_dict(InfileNameFN):
    Infile = open(InfileNameFN, "r")
    seqDictFN = OrderedDict()
    for line in Infile:
        line = line.rstrip("\n")
        if line.startswith("    <span class=sel>"):
            name, seq = re.split(r"</span> {2,}", line)
            name = re.sub(" +<span class=sel>", "", name)
            name = re.sub(" +$", "", name)
            hitNames = [ele for ele in seqDictFN.keys() if name == ele]
            if not hitNames:
                seqDictFN[name] = ""
            seqDictFN[name] += seq
    Infile.close()
    return seqDictFN

def trimalOutToSharp(recsFN):
    for name, seq in recsFN.items():
        seq = re.sub(r"<span class=sel>-</span>",    "-", seq)
        seq = re.sub(r"<span class=sel>X</span>",    "X", seq)
        seq = re.sub(r"<span class=sel>(\w)</span>", "#", seq)
        recsFN[name] = seq
    return recsFN


def trimOutSelectedAASite(recsFN):
    firstSeq = list(recsFN.values())[0]
    selectedSitesSR = []
    for i in range(len(firstSeq)):
        if firstSeq[i] == "#":
            selectedSitesSR.append(i) 
    return selectedSitesSR;


def readFasta_dict(InfileNameFN):
    Infile = open(InfileNameFN, "r")
    seqDictFN  = OrderedDict()
    for Line in Infile:
        Line = Line.rstrip("\n")
        if Line[0] == ">":
            Name            = Line
            seqDictFN[Name] = ""
        else:
            seqDictFN[Name] += Line
    Infile.close()
    return seqDictFN


def codonSelect(sequenceSR, selectedAAsites):
    selectedCodons = ""
    for AAsite in selectedAAsites:
        DNAsite = AAsite * 3;
        codon = sequenceSR[DNAsite:DNAsite+3]
        selectedCodons += codon
    return selectedCodons

def cDNAsiteSelect (recsCDNA, selectedAAsites):
    seqDictFN  = OrderedDict()
    for name, sequence in recsCDNA.items():
        sequenceSelected = codonSelect(sequence, selectedAAsites);
        seqDictFN[name]  = sequenceSelected;
    return seqDictFN

def trimal_cDNAcleaner (htmlInfile, cdnaInfile):

    recsTrimOut     = readTrimalOut_dict(htmlInfile)
    recsAA          = trimalOutToSharp(recsTrimOut)
    selectedAAsites = trimOutSelectedAASite(recsAA)

    recsCDNA = readFasta_dict(cdnaInfile)
    recsCDNA = cDNAsiteSelect(recsCDNA, selectedAAsites);

    return recsCDNA

### Main

recsRes = trimal_cDNAcleaner("infile_AA.html", "infile_Nuc.txt")
for name, seq in recsRes.items():
    print (name)
    print (seq)
